/******************************************************************************
 * The AnimatedView class is where the action is.  I have overridden the
 * onDraw method to do the real work  
 * 
 * Written by John Cole at The University of Texas at Dallas.
 ******************************************************************************/
package com.example.paddleball;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.drawable.BitmapDrawable;
import android.media.AudioFormat;
import android.media.AudioManager;
import android.media.AudioTrack;
import android.os.Handler;
import android.util.AttributeSet;
import android.widget.ImageView;

public class AnimatedView extends ImageView
{
	private Context mContext;
	int x = -1;
	int y = -1;
	private int xVelocity = 12;
	private int yVelocity = -6;
	private Handler h;
	private int frameRate = 20;
	private int paddleX = 0;
	private int paddleY = 0;
	private int paddleWidth = 0;
	private int paddleHeight = 20 ;
	private int courtTop;
	private Rect paddle;
	private Paint paddleColor;
	private Paint textColor;
	private boolean bInMotion = false;
	private int score = 0;
	private int prevScore = 0;
	boolean bHit = false;
	
	// Variables for sound.
  private final float duration = 0.3f; // seconds
  private final int sampleRate = 8000;
  private final int numSamples = (int)(duration * sampleRate);
  private final double sample[] = new double[numSamples];
  private final double freqOfTone = 220; // hz
  private final byte generatedSnd[] = new byte[2 * numSamples];

	/****************************************************************************
	 * This constructor takes a context and a set of attributes and calls the
	 * ImageView's constructor.  Then do our initialization.
	 ****************************************************************************/
	public AnimatedView(Context context, AttributeSet attrs)
	{
		super(context, attrs);
		mContext = context;
		h = new Handler();
		paddleColor = new Paint();
		paddleColor.setColor(Color.GREEN);
		paddle = new Rect();
		textColor = new Paint();
		textColor.setColor(Color.WHITE);
	}

	/****************************************************************************
	 * Set the paddle's X position.
	 ****************************************************************************/
  public void setPaddle(int px)
	{
		paddleX = px;
	}
  
  public void setSpeed(int speed)
  {
  	frameRate = speed;
  }
  
  public int getCourtTop()
  {
  	return courtTop;
  }
	
	/****************************************************************************
	 * If the ball isn't moving, start it and set the score to zero.
	 ****************************************************************************/
	public void setMotion()
	{
		if (!bInMotion)
			{
			bInMotion = true;
			score = 0;
			}
	}

	/****************************************************************************
	 * Show the score above the line.
	 ****************************************************************************/
	private void showScore(Canvas c)
	{
		textColor.setTextSize(30);
		String str = "Professor Cole's Pong.";
		c.drawText(str, 0, 30, textColor);
		str = "Score: " + score;
		float w = textColor.measureText(str);
		c.drawText(str,  c.getWidth() - (int)w, 30, textColor);
		str = "Previous core: " + prevScore;
		w = textColor.measureText(str);
		c.drawText(str,  c.getWidth() - (int)w, 65, textColor);
	}
	
	/****************************************************************************
	 * Create a new thread that will let us time the rate of the ball.
	 ****************************************************************************/
  private Runnable r = new Runnable() {
		@Override
		public void run()
		{
			invalidate();
		}
	};

	/****************************************************************************
	 * Draw everything.  It may seem wasteful to re-draw the entire screen with
	 * every frame refresh, but it works.  Get the ball from our resources, set
	 * up the paddle, and draw green lines around the court.
	 ****************************************************************************/
	protected void onDraw(Canvas c)
	{
		BitmapDrawable ball = (BitmapDrawable) mContext.getResources().getDrawable(
		    R.drawable.ic_launcher);
		int ballHeight = ball.getBitmap().getHeight();
		int ballWidth = ball.getBitmap().getWidth();
		paddleY = this.getHeight() - 100 - paddleHeight;
		paddleWidth = this.getWidth() / 5;
		courtTop = this.getHeight() -this.getWidth();
		// Draw green lines around the court.
		c.drawLine(0,  courtTop, this.getWidth(), courtTop, paddleColor);
		c.drawLine(0,  courtTop, 0, this.getHeight(), paddleColor);
		c.drawLine(this.getWidth()-1, courtTop, this.getWidth()-1, this.getHeight(), paddleColor);
		c.drawLine(0,  this.getHeight()-1, this.getWidth(), this.getHeight()-1, paddleColor);
		
	  paddleColor.setTextSize(30);
//???    c.drawText("x: " + x + "  y=" + y + "  ballwidth: " + ballWidth + "  PaddleX: " + paddleX, 20, 20, paint);
		
		// If the Y position of the ball is past the paddle and the x position is
		// not within the range of the paddle, the ball has been dropped.
		if ((y - ballHeight / 2 + Math.abs(yVelocity) > paddleY) && (x < paddleX || x > paddleX + paddleWidth) )
		{
			bInMotion = false;
			yVelocity = Math.abs(yVelocity) * -1;
			xVelocity = Math.abs(xVelocity);
			prevScore = score;
		}
		
		// If the Y position of the ball is within 10 pixels of the paddle and the X
		// position of the ball is within the range of the paddle, reverse the Y
		// direction of the ball.
		if (y > paddleY - ballHeight && y < paddleY + paddleHeight)
			{
//???			c.drawText("YHit", 300, 20, paddleColor);
			if (x >= paddleX && x <= paddleX + paddleWidth - ballWidth / 2)
				{
//???					c.drawText("Hit", 300, 20, paddleColor);
					yVelocity = yVelocity * -1;
					score++;
					bHit = true; 
				}
			}
		
		// If the ball isn't moving and it isn't on top of the paddle, put it
		// there and center the paddle and ball.
		if (!bInMotion)
		{
			if (x - ballHeight == paddleY)
				return;
			else
			{
				y = paddleY - ballHeight;
				x = this.getWidth() / 2 - ballWidth / 2;
				paddleX = this.getWidth() / 2 - paddleWidth / 2;
			}
		}
		// The ball is moving.  Let's see if it hit anything.
		else
		{
			// If the ball has not been initialized, center it.
			if (x < 0 && y < 0)
			{
				x = this.getWidth() / 2;
				y = this.getHeight() / 2;
			}
			else
			{
				// Move the ball.  If it hits a wall or the ceiling, reverse the
				// appropriate direction.
				x += xVelocity;
				y += yVelocity;
				if ((x > this.getWidth() - ballWidth) || (x < 0))
				{
					xVelocity = xVelocity * -1;
				}
				if ((y > this.getHeight() - ballHeight) || (y < courtTop))
				{
					yVelocity = yVelocity * -1;
				}
			}
		}
		// Having recalculated various positions, display the ball and paddle.
		c.drawBitmap(ball.getBitmap(), x, y, null);
		paddle.top = paddleY;
		paddle.left = paddleX;
		paddle.right = paddleX + paddleWidth;
		paddle.bottom = paddleY + paddleHeight;
		c.drawRect(paddle, paddleColor);
		h.postDelayed(r, frameRate);
		showScore(c);
	}
	
  void genTone(){
    // fill out the array
    for (int i = 0; i < numSamples; ++i) {
        sample[i] = Math.sin(2 * Math.PI * i / (sampleRate/freqOfTone));
    }

    // convert to 16 bit pcm sound array
    // assumes the sample buffer is normalized.
    int idx = 0;
    for (final double dVal : sample) {
        // scale to maximum amplitude
        final short val = (short) ((dVal * 32767));
        // in 16 bit wav PCM, first byte is the low order byte
        generatedSnd[idx++] = (byte) (val & 0x00ff);
        generatedSnd[idx++] = (byte) ((val & 0xff00) >>> 8);

    }
}	
  
  
  void playSound(){
    final AudioTrack audioTrack = new AudioTrack(AudioManager.STREAM_MUSIC,
            sampleRate, AudioFormat.CHANNEL_OUT_MONO,
            AudioFormat.ENCODING_PCM_16BIT, generatedSnd.length,
            AudioTrack.MODE_STATIC);
    audioTrack.write(generatedSnd, 0, generatedSnd.length);
    audioTrack.play();
}  
  
  // Use a new tread as this can take a while
  Thread soundThread = new Thread(new Runnable() {
      public void run() {
          genTone();
          h.post(new Runnable() {

              public void run() {
              	if (bHit)
              	{
                  playSound();
                  bHit = false;
              	}
              }
          });
      }
  });


  
}

